@@ -1,8 +1,8 @@
 package Cassandra::Client::Pool;
 our $AUTHORITY = 'cpan:TVDW';
-$Cassandra::Client::Pool::VERSION = '0.13_004'; # TRIAL
+$Cassandra::Client::Pool::VERSION = '0.13_005'; # TRIAL
 
-$Cassandra::Client::Pool::VERSION = '0.13004';use 5.010;
+$Cassandra::Client::Pool::VERSION = '0.13005';use 5.010;
 use strict;
 use warnings;
 
@@ -47,6 +47,7 @@ sub init {
     $self->{policy}{datacenter} ||= $first_connection->{datacenter};
 
     $self->add($first_connection);
+    $self->{policy}->set_connecting($first_connection->ip_address);
     $self->{policy}->set_connected($first_connection->ip_address);
 
     # Master selection, warmup, etc
@@ -194,6 +195,7 @@ sub connect_if_needed {
 
             if ($done == $expect) {
                 $callback->() if $callback;
+                undef $callback;
             }
         });
     }
@@ -217,7 +219,7 @@ sub spawn_new_connection {
     );
 
     $self->{connecting}{$host}= $connection;
-    $self->{policy}->set_connected($host);
+    $self->{policy}->set_connecting($host);
 
     $connection->connect(sub {
         my ($error)= @_;
@@ -246,6 +248,8 @@ sub spawn_new_connection {
 
             $self->connect_if_needed;
         } else {
+            $self->{policy}->set_connected($host);
+
             $self->add($connection);
         }
 
@@ -293,7 +297,7 @@ Cassandra::Client::Pool
 
 =head1 VERSION
 
-version 0.13_004
+version 0.13_005
 
 =head1 AUTHOR
 
